'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _updateCounter = require('./updateCounter');

var _proxies = require('./reconciler/proxies');

function resolveType(type, disableComponentProxy) {
  // We only care about composite components
  if (typeof type !== 'function') {
    return type;
  }

  // always could...
  const couldWrapWithProxy = true;

  // is proxing is disabled - do not create auto proxies, but use the old ones
  const proxy = !disableComponentProxy && couldWrapWithProxy ? (0, _proxies.createProxyForType)(type) : (0, _proxies.getProxyByType)(type);

  if (!proxy) {
    return type;
  }

  return proxy.get();
}

const reactHotLoader = {
  register(type, uniqueLocalName, fileName) {
    (0, _updateCounter.didUpdate)();

    if (typeof type !== 'function') {
      return;
    }
    if (!uniqueLocalName || !fileName) {
      return;
    }
    if (typeof uniqueLocalName !== 'string' || typeof fileName !== 'string') {
      return;
    }
    const id = fileName + '#' + uniqueLocalName; // eslint-disable-line prefer-template

    (0, _proxies.updateProxyById)(id, type);
  },

  reset(useWeakMap) {
    (0, _proxies.resetProxies)(useWeakMap);
  },

  patch(React) {
    if (!React.createElement.isPatchedByReactHotLoader) {
      const originalCreateElement = React.createElement;
      React.createElement = (type, ...args) => {
        // Trick React into rendering a proxy so that
        // its state is preserved when the class changes.
        // This will update the proxy if it's for a known type.
        const resolvedType = resolveType(type, reactHotLoader.disableComponentProxy);
        return originalCreateElement(resolvedType, ...args);
      };
      React.createElement.isPatchedByReactHotLoader = true;
    }

    if (!React.createFactory.isPatchedByReactHotLoader) {
      React.createFactory = type => {
        // Patch React.createFactory to use patched createElement
        // because the original implementation uses the internal,
        // unpatched ReactElement.createElement
        const factory = React.createElement.bind(null, type);
        factory.type = type;
        return factory;
      };
      React.createFactory.isPatchedByReactHotLoader = true;
    }

    if (!React.Children.only.isPatchedByReactHotLoader) {
      const originalChildrenOnly = React.Children.only;
      React.Children.only = element => originalChildrenOnly(_extends({}, element, { type: resolveType(element.type) }));
      React.Children.only.isPatchedByReactHotLoader = true;
    }

    reactHotLoader.reset();
  },

  disableComponentProxy: false,

  config: {
    logLevel: 'error'
  }
};

exports.default = reactHotLoader;